#ifndef PRINTER_HEADER
#define PRINTER_HEADER

#include "Absyn.H"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* Certain applications may improve performance by changing the buffer size */
#define BUFFER_INITIAL 2000
/* You may wish to change _L_PAREN or _R_PAREN */
#define _L_PAREN '('
#define _R_PAREN ')'

class PrintAbsyn : public Visitor
{
 protected:
  int _n_, _i_;
  /* The following are simple heuristics for rendering terminals */
  /* You may wish to change them */
  void render(Char c);
  void render(String s);
  void indent(void);
  void backup(void);
 public:
  PrintAbsyn(void);
  ~PrintAbsyn(void);
  char* print(Visitable* v);

  void visitBenchmarkC(BenchmarkC *p); /* abstract class */
  void visitBenchmark(Benchmark *p);
  void visitAttribute(Attribute *p); /* abstract class */
  void visitLogicAttribute(LogicAttribute *p);
  void visitFamilyAttribute(FamilyAttribute *p);
  void visitAssumptionAttribute(AssumptionAttribute *p);
  void visitFormulaAttribute(FormulaAttribute *p);
  void visitStatusAttribute(StatusAttribute *p);
  void visitExtraSortsAttribute(ExtraSortsAttribute *p);
  void visitExtraFunsAttribute(ExtraFunsAttribute *p);
  void visitNotesAttribute(NotesAttribute *p);
  void visitAnnotationAttribute(AnnotationAttribute *p);
  void visitListAttribute(ListAttribute* p);
  void visitStatus(Status *p); /* abstract class */
  void visitSatStatus(SatStatus *p);
  void visitUnSatStatus(UnSatStatus *p);
  void visitUnknownStatus(UnknownStatus *p);
  void visitSort(Sort *p); /* abstract class */
  void visitIdentSort(IdentSort *p);
  void visitCompositeSort(CompositeSort *p);
  void visitListSort(ListSort* p);
  void visitSortDeclC(SortDeclC *p); /* abstract class */
  void visitSortDecl(SortDecl *p);
  void visitSortSynonymDecl(SortSynonymDecl *p);
  void visitListSortDeclC(ListSortDeclC* p);
  void visitTerm(Term *p); /* abstract class */
  void visitConstantTerm(ConstantTerm *p);
  void visitNullaryTerm(NullaryTerm *p);
  void visitFunctionTerm(FunctionTerm *p);
  void visitLetTerm(LetTerm *p);
  void visitQuantifierTerm(QuantifierTerm *p);
  void visitAnnotationTerm(AnnotationTerm *p);
  void visitListTerm(ListTerm* p);
  void visitBindingC(BindingC *p); /* abstract class */
  void visitBinding(Binding *p);
  void visitListBindingC(ListBindingC* p);
  void visitQuantifier(Quantifier *p); /* abstract class */
  void visitAllQuantifier(AllQuantifier *p);
  void visitExQuantifier(ExQuantifier *p);
  void visitSymbolRef(SymbolRef *p); /* abstract class */
  void visitIdentifierRef(IdentifierRef *p);
  void visitCastIdentifierRef(CastIdentifierRef *p);
  void visitFunctionDeclC(FunctionDeclC *p); /* abstract class */
  void visitConstantDecl(ConstantDecl *p);
  void visitMetaConstantDecl(MetaConstantDecl *p);
  void visitFunctionDecl(FunctionDecl *p);
  void visitMacroDecl(MacroDecl *p);
  void visitListFunctionDeclC(ListFunctionDeclC* p);
  void visitSortedVariableC(SortedVariableC *p); /* abstract class */
  void visitSortedVariable(SortedVariable *p);
  void visitListSortedVariableC(ListSortedVariableC* p);
  void visitSpecConstant(SpecConstant *p); /* abstract class */
  void visitNumConstant(NumConstant *p);
  void visitRatConstant(RatConstant *p);
  void visitHexConstant(HexConstant *p);
  void visitBinConstant(BinConstant *p);
  void visitStringConstant(StringConstant *p);
  void visitMetaConstant(MetaConstant *p); /* abstract class */
  void visitNumMetaConstant(NumMetaConstant *p);
  void visitRatMetaConstant(RatMetaConstant *p);
  void visitHexMetaConstant(HexMetaConstant *p);
  void visitBinMetaConstant(BinMetaConstant *p);
  void visitStringMetaConstant(StringMetaConstant *p);
  void visitIdentifier(Identifier *p); /* abstract class */
  void visitSymbolIdent(SymbolIdent *p);
  void visitIndexIdent(IndexIdent *p);
  void visitIndexC(IndexC *p); /* abstract class */
  void visitIndex(Index *p);
  void visitListIndexC(ListIndexC* p);
  void visitAnnotation(Annotation *p); /* abstract class */
  void visitAttrAnnotation(AttrAnnotation *p);
  void visitListAnnotation(ListAnnotation* p);
  void visitAttrParam(AttrParam *p); /* abstract class */
  void visitSomeAttrParam(SomeAttrParam *p);
  void visitNoAttrParam(NoAttrParam *p);
  void visitSExpr(SExpr *p); /* abstract class */
  void visitConstantSExpr(ConstantSExpr *p);
  void visitSymbolSExpr(SymbolSExpr *p);
  void visitParenSExpr(ParenSExpr *p);
  void visitListSExpr(ListSExpr* p);

  void visitInteger(Integer i);
  void visitDouble(Double d);
  void visitChar(Char c);
  void visitString(String s);
  void visitIdent(String s);
 protected:
  void inline bufAppend(const char* s)
  {
    int len = strlen(s);
    while (cur_ + len > buf_size)
    {
      buf_size *= 2; /* Double the buffer size */
      resizeBuffer();
    }
    for(int n = 0; n < len; n++)
    {
      buf_[cur_ + n] = s[n];
    }
    cur_ += len;
    buf_[cur_] = 0;
  }
  void inline bufAppend(const char c)
  {
    if (cur_ == buf_size)
    {
      buf_size *= 2; /* Double the buffer size */
      resizeBuffer();
    }
    buf_[cur_] = c;
    cur_++;
    buf_[cur_] = 0;
  }
  void inline bufReset(void)
  {
    cur_ = 0;
    buf_size = BUFFER_INITIAL;
    resizeBuffer();
    memset(buf_, 0, buf_size);
  }
  void inline resizeBuffer(void)
  {
    char* temp = (char*) malloc(buf_size);
    if (!temp)
    {
      fprintf(stderr, "Error: Out of memory while attempting to grow buffer!\n");
      exit(1);
    }
    if (buf_)
    {
      strcpy(temp, buf_);
      free(buf_);
    }
    buf_ = temp;
  }
  char *buf_;
  int cur_, buf_size;
};



class ShowAbsyn : public Visitor
{
 public:
  ShowAbsyn(void);
  ~ShowAbsyn(void);
  char* show(Visitable* v);

  void visitBenchmarkC(BenchmarkC *p); /* abstract class */
  void visitBenchmark(Benchmark *p);
  void visitAttribute(Attribute *p); /* abstract class */
  void visitLogicAttribute(LogicAttribute *p);
  void visitFamilyAttribute(FamilyAttribute *p);
  void visitAssumptionAttribute(AssumptionAttribute *p);
  void visitFormulaAttribute(FormulaAttribute *p);
  void visitStatusAttribute(StatusAttribute *p);
  void visitExtraSortsAttribute(ExtraSortsAttribute *p);
  void visitExtraFunsAttribute(ExtraFunsAttribute *p);
  void visitNotesAttribute(NotesAttribute *p);
  void visitAnnotationAttribute(AnnotationAttribute *p);
  void visitListAttribute(ListAttribute* p);
  void visitStatus(Status *p); /* abstract class */
  void visitSatStatus(SatStatus *p);
  void visitUnSatStatus(UnSatStatus *p);
  void visitUnknownStatus(UnknownStatus *p);
  void visitSort(Sort *p); /* abstract class */
  void visitIdentSort(IdentSort *p);
  void visitCompositeSort(CompositeSort *p);
  void visitListSort(ListSort* p);
  void visitSortDeclC(SortDeclC *p); /* abstract class */
  void visitSortDecl(SortDecl *p);
  void visitSortSynonymDecl(SortSynonymDecl *p);
  void visitListSortDeclC(ListSortDeclC* p);
  void visitTerm(Term *p); /* abstract class */
  void visitConstantTerm(ConstantTerm *p);
  void visitNullaryTerm(NullaryTerm *p);
  void visitFunctionTerm(FunctionTerm *p);
  void visitLetTerm(LetTerm *p);
  void visitQuantifierTerm(QuantifierTerm *p);
  void visitAnnotationTerm(AnnotationTerm *p);
  void visitListTerm(ListTerm* p);
  void visitBindingC(BindingC *p); /* abstract class */
  void visitBinding(Binding *p);
  void visitListBindingC(ListBindingC* p);
  void visitQuantifier(Quantifier *p); /* abstract class */
  void visitAllQuantifier(AllQuantifier *p);
  void visitExQuantifier(ExQuantifier *p);
  void visitSymbolRef(SymbolRef *p); /* abstract class */
  void visitIdentifierRef(IdentifierRef *p);
  void visitCastIdentifierRef(CastIdentifierRef *p);
  void visitFunctionDeclC(FunctionDeclC *p); /* abstract class */
  void visitConstantDecl(ConstantDecl *p);
  void visitMetaConstantDecl(MetaConstantDecl *p);
  void visitFunctionDecl(FunctionDecl *p);
  void visitMacroDecl(MacroDecl *p);
  void visitListFunctionDeclC(ListFunctionDeclC* p);
  void visitSortedVariableC(SortedVariableC *p); /* abstract class */
  void visitSortedVariable(SortedVariable *p);
  void visitListSortedVariableC(ListSortedVariableC* p);
  void visitSpecConstant(SpecConstant *p); /* abstract class */
  void visitNumConstant(NumConstant *p);
  void visitRatConstant(RatConstant *p);
  void visitHexConstant(HexConstant *p);
  void visitBinConstant(BinConstant *p);
  void visitStringConstant(StringConstant *p);
  void visitMetaConstant(MetaConstant *p); /* abstract class */
  void visitNumMetaConstant(NumMetaConstant *p);
  void visitRatMetaConstant(RatMetaConstant *p);
  void visitHexMetaConstant(HexMetaConstant *p);
  void visitBinMetaConstant(BinMetaConstant *p);
  void visitStringMetaConstant(StringMetaConstant *p);
  void visitIdentifier(Identifier *p); /* abstract class */
  void visitSymbolIdent(SymbolIdent *p);
  void visitIndexIdent(IndexIdent *p);
  void visitIndexC(IndexC *p); /* abstract class */
  void visitIndex(Index *p);
  void visitListIndexC(ListIndexC* p);
  void visitAnnotation(Annotation *p); /* abstract class */
  void visitAttrAnnotation(AttrAnnotation *p);
  void visitListAnnotation(ListAnnotation* p);
  void visitAttrParam(AttrParam *p); /* abstract class */
  void visitSomeAttrParam(SomeAttrParam *p);
  void visitNoAttrParam(NoAttrParam *p);
  void visitSExpr(SExpr *p); /* abstract class */
  void visitConstantSExpr(ConstantSExpr *p);
  void visitSymbolSExpr(SymbolSExpr *p);
  void visitParenSExpr(ParenSExpr *p);
  void visitListSExpr(ListSExpr* p);

  void visitInteger(Integer i);
  void visitDouble(Double d);
  void visitChar(Char c);
  void visitString(String s);
  void visitIdent(String s);
 protected:
  void inline bufAppend(const char* s)
  {
    int len = strlen(s);
    while (cur_ + len > buf_size)
    {
      buf_size *= 2; /* Double the buffer size */
      resizeBuffer();
    }
    for(int n = 0; n < len; n++)
    {
      buf_[cur_ + n] = s[n];
    }
    cur_ += len;
    buf_[cur_] = 0;
  }
  void inline bufAppend(const char c)
  {
    if (cur_ == buf_size)
    {
      buf_size *= 2; /* Double the buffer size */
      resizeBuffer();
    }
    buf_[cur_] = c;
    cur_++;
    buf_[cur_] = 0;
  }
  void inline bufReset(void)
  {
    cur_ = 0;
    buf_size = BUFFER_INITIAL;
    resizeBuffer();
    memset(buf_, 0, buf_size);
  }
  void inline resizeBuffer(void)
  {
    char* temp = (char*) malloc(buf_size);
    if (!temp)
    {
      fprintf(stderr, "Error: Out of memory while attempting to grow buffer!\n");
      exit(1);
    }
    if (buf_)
    {
      strcpy(temp, buf_);
      free(buf_);
    }
    buf_ = temp;
  }
  char *buf_;
  int cur_, buf_size;
};



#endif

