/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class to read, instantiate, and output snippets of some output format
 * (here in particular used to instantiate SMT-LIB axioms)
 */

#ifndef FILE_TEMPLATE
#define FILE_TEMPLATE 1

#include <iostream>
#include <string>
#include <vector>
#include <map>

typedef std::map<std::string, std::string> StringSubstitution;

class FileTemplate {
private:
    const std::string filename;
    std::vector<std::string>* contents;
    bool* loaded;

    void load() const;

    /**
     * Replace every occurrence of the string "find" in "line" with the string
     * "replace"
     */
    std::string replace(const std::string& line,
                        const std::string& find, const std::string& replace) const;

    /**
     * Apply a string substitution to a string
     */
    std::string replace(const std::string& line, const StringSubstitution& subst) const;

public:
    FileTemplate(const std::string& filename);

    void instantiate(std::ostream& os, const StringSubstitution& subst) const;
};

/**
 * Exception that is raised when reading a file template fails
 */
class IOException : public std::exception {
private:
    const std::string message;

public:
    IOException(const std::string& message);
    ~IOException() throw ();

    const char* what() const throw();
};

#endif
