/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "id-lineariser.H"

#include <sstream>

namespace SMTBackend {

using namespace std;

void IdLineariser::linearise(ostream& stream, const IndexedString& name) {
    stream << makePrintable(name.body);
    for (vector<string>::const_iterator it = name.indexes.begin();
         it != name.indexes.end();
         ++it)
        stream << "_" << *it;
}

void IdLineariser::linearise(ostream& stream, const SortPtr& sort) {
    linearise(stream, sort->op->name);
    for (vector<SortPtr>::const_iterator it = sort->arguments.begin();
         it != sort->arguments.end();
         ++it) {
        stream << "_";
        linearise(stream, *it);
    }
}

std::string IdLineariser::toString(const SortPtr& sort) {
    stringstream res;
    linearise(res, sort);
    return res.str();
}

std::string IdLineariser::makePrintable(const std::string& name) {
    assert(!name.empty());

    map<string, string>::iterator pos = nameTranslation.find(name);
    if (pos != nameTranslation.end())
        return pos->second;

    stringstream newNameStream;
    for (unsigned int i = 0; i < name.size(); ++i) {
        const char c = name[i];
        if (PRINTABLE_CHARS.find(c) == string::npos)
            newNameStream << '_';
        else
            newNameStream << c;
    }

    const string cleanName = newNameStream.str();
    if (cleanName == name && LETTERS.find(cleanName[0]) != string::npos) {
        nameTranslation[name] = name;
        usedNames.insert(name);
        return name;
    }

    const string baseName = "preproc_" + cleanName;

    string nameCandidate = baseName;
    unsigned int counter = 0;
    while (usedNames.find(nameCandidate) != usedNames.end()) {
        stringstream str;
        str << baseName << '_' << counter;
        counter = counter + 1;
        nameCandidate = str.str();
    }

    nameTranslation[name] = nameCandidate;
    usedNames.insert(nameCandidate);
    return nameCandidate;
}

const std::string IdLineariser::LETTERS =
    "aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ";
const std::string IdLineariser::PRINTABLE_CHARS =
    IdLineariser::LETTERS + "0123456789.'_";

}
