/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Functionality to linearise identifiers, sorts, etc.
 */

#ifndef SMT_ID_LINEARISER
#define SMT_ID_LINEARISER 1

#include "../smtlib-absyn.H"

#include <iostream>
#include <set>
#include <map>
#include <string>

namespace SMTBackend {

using namespace SMTLibAbsyn;

class IdLineariser {

private:
    /**
     * Names of sorts, variables, and functions have to be sanitised, because
     * SMT-LIB 1 is stricter than SMT-LIB 2 concerning the characters allowed
     * in identifiers
     */
    std::map<std::string, std::string> nameTranslation;
    std::set<std::string> usedNames;

    const static std::string LETTERS;
    const static std::string PRINTABLE_CHARS;

public:

    void linearise(std::ostream& stream, const IndexedString& name);
    void linearise(std::ostream& stream, const SortPtr& sort);

    std::string toString(const SortPtr& sort);

    std::string makePrintable(const std::string& name);
};

}

#endif
