/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Declarations and axioms for the theory of finite relations with cardinality.
 * These axioms cannot be loaded from template files (unlike the axioms of the
 * other theories, like sets) because the type constructors and functions have
 * an unbounded number of arguments (and cannot be represented using
 * parametric polymorphism).
 */

#ifndef SMT_RELATION_AXIOMS
#define SMT_RELATION_AXIOMS 1

#include "../theories/relations.H"
#include "../smtlib-absyn.H"
#include "id-lineariser.H"

#include <iostream>

namespace SMTBackend {

using namespace SMTLibAbsyn;

class RelationAxiomGenerator {

private:
    std::ostream& os;
    IdLineariser idLineariser;

    /**
     * Linearise the name of the tuple constructor for the given tuple sort
     */
    void tuple(const SortPtr& sort);

    /**
     * Linearise the name of the projection operator for the given tuple sort
     * and index
     */
    void project(const SortPtr& sort, unsigned int index);

    /**
     * Linearise the name of the product operator for the given tuple sort
     * and index
     */
    void product(const SortPtr& sort);

    void startComment(const SortPtr& sort, const std::string& desc);
    void stopComment(const SortPtr& sort, const std::string& desc);

public:
    RelationAxiomGenerator(std::ostream& os);

    void instantiateTupleAxioms(const SortPtr& sort);
    void instantiateSetTupleAxioms(const SortPtr& sort);
};

}

#endif
