/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <vector>

#include "smtlib-absyn.H"
#include "smtlib-absyn-visitors.H"

namespace SMTLibAbsyn {

using namespace std;
using namespace Visitors;

/**
 * Visitor that checks whether the specified types of bound variables are
 * correct. The first argument of the visitor (int) is used to store the number
 * of binders that were passed on the way to a variable occurrence.
 */
class PreTypeChecker : public DefaultExpressionVisitor<int, int, PreVisitResultPtr> {
private:
    const vector<SortPtr>& variableSorts;

public:
    PreTypeChecker(const vector<SortPtr>& variableSorts)
        : variableSorts(variableSorts) {}

    void defaultAction(const ExpressionPtr& node) {
        // nothing else to be done
        res = keepArg();
    }

    void visitVariable(const CountingPtr<Variable>& node) {
        int realIndex = (int)node->index - arg0;
        if (realIndex >= 0 && realIndex < (int)variableSorts.size())
            assert(*node->sort() ==
                   *variableSorts[(int)variableSorts.size() - realIndex - 1]);

        res = keepArg();
    }

    void visitLetBinder(const CountingPtr<LetBinder>& node) {
        vector<int> newArgs (node->variableNames.size(), arg0);
        newArgs.push_back(arg0 + node->variableNames.size());

        res = subArgs(newArgs);
    }

    void visitQuantifiedExpression(const CountingPtr<QuantifiedExpression>& node) {
        res = uniSubArgs((int)(arg0 + node->variableNames.size()));
    }
};

void Binder::checkVariableTypes(const vector<SortPtr>& variableSorts,
                                const ExpressionPtr& body) {
    PreTypeChecker preTypeChecker (variableSorts);
    PassiveExpressionVisitor<int, const std::vector<int>*, int> postTypeChecker;

    collectingVisit(body, 0, preTypeChecker, postTypeChecker);
}

}
