/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The empty theory, providing propositional logic and standard operations
 * available on all types (equality, ite).
 */

#ifndef THEORY_EMPTY
#define THEORY_EMPTY 1

#include "theory.H"
#include "../smtlib-absyn.H"
#include "../smtlib-resolver.H"
#include "../multi-arity-ops.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class Empty;

class TheoryHelpers : public Theory {
protected:
    void registerMonoFunction(Environment &env,
                              const MonoFunctionPtr& function) const;
    void registerPolyFunction(Environment &env,
                              const PolyFunctionFamilyPtr& function) const;
    void registerMultiArityFunction(Environment &env,
                                    const MonoFunctionPtr& basisFunction,
                                    const MultiArityFunctionFamilyPtr& function) const;
    void registerMultiArityFunction(Environment &env,
                                    const PolyFunctionFamilyPtr& basisFunction,
                                    const MultiArityFunctionFamilyPtr& function) const;

public:
    virtual const Empty& getEmpty() const = 0;
};

////////////////////////////////////////////////////////////////////////////////

/**
 * Function family for the "distinct" operator
 */
class DistinctFunction : public FunctionFamily {
protected:
    // We just use a vector to store the created instances
    // TODO: replace this with a more efficient map
    std::vector<MonoFunctionPtr> instances;

public:
    MonoFunctionPtr
        lookup(const std::vector<std::string>& indexes,
               const std::vector<SortPtr>& argumentSorts);
    MonoFunctionPtr
        lookup(const std::vector<std::string>& indexes,
               const SortPtr& resSort,
               const std::vector<SortPtr>& argumentSorts);
};

typedef CountingPtr<DistinctFunction> DistinctFunctionPtr;

////////////////////////////////////////////////////////////////////////////////

class Empty : public TheoryHelpers {
public:
    Empty();

    const SortCtorPtr           sort_Bool;
    const MonoFunctionPtr       fun_true;
    const MonoFunctionPtr       fun_false;
    const MonoFunctionPtr       fun_not;
    const MonoFunctionPtr       fun_implies;
    const MonoFunctionPtr       fun_and;
    const MonoFunctionPtr       fun_or;
    const MonoFunctionPtr       fun_xor;
    const PolyFunctionFamilyPtr polyfun_equal;
    const PolyFunctionFamilyPtr polyfun_ite;
    const DistinctFunctionPtr   polyfun_distinct;

    const MultiArityFunctionFamilyPtr multiArityFun_implies;
    const MultiArityFunctionFamilyPtr multiArityFun_and;
    const MultiArityFunctionFamilyPtr multiArityFun_or;
    const MultiArityFunctionFamilyPtr multiArityFun_xor;
    const MultiArityFunctionFamilyPtr multiArityFun_equal;

    void registerSymbols(Environment &env) const;
    void registerSymbolsLocally(Environment &env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
};

}

#endif
