/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The theory of finite maps
 */

#ifndef THEORY_MAPS
#define THEORY_MAPS 1

#include "sets.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class Maps : public TheoryHelpers {
public:
    Maps(const Empty& empty, const Sets& sets);

    const Empty& empty;
    const Sets& sets;

    const SortCtorPtr           sort_Map;
    const PolyFunctionFamilyPtr polyfun_emptyMap;
    const PolyFunctionFamilyPtr polyfun_apply;
    const PolyFunctionFamilyPtr polyfun_overwrite;
    const PolyFunctionFamilyPtr polyfun_domain;
    const PolyFunctionFamilyPtr polyfun_range;
    const PolyFunctionFamilyPtr polyfun_restrict;
    const PolyFunctionFamilyPtr polyfun_subtract;

    void registerSymbols(Environment& env) const;
    void registerSymbolsLocally(Environment& env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
    static Maps* create();
};

}

#endif
