/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The theory of finite sets with cardinality
 */

#ifndef THEORY_SETS
#define THEORY_SETS 1

#include "integers.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class Sets : public TheoryHelpers {
public:
    Sets(const Empty& empty, const Integers& integers);

    const Empty& empty;
    const Integers& integers;

    const SortCtorPtr           sort_Set;
    const PolyFunctionFamilyPtr polyfun_emptySet;
    const PolyFunctionFamilyPtr polyfun_insert;
    const PolyFunctionFamilyPtr polyfun_in;
    const PolyFunctionFamilyPtr polyfun_subset;
    const PolyFunctionFamilyPtr polyfun_union;
    const PolyFunctionFamilyPtr polyfun_inter;
    const PolyFunctionFamilyPtr polyfun_setminus;
    const PolyFunctionFamilyPtr polyfun_card;

    const MultiArityFunctionFamilyPtr multiArityFun_insert;
    const MultiArityFunctionFamilyPtr multiArityFun_subset;
    const MultiArityFunctionFamilyPtr multiArityFun_union;
    const MultiArityFunctionFamilyPtr multiArityFun_inter;
    const MultiArityFunctionFamilyPtr multiArityFun_setminus;

    void registerSymbols(Environment& env) const;
    void registerSymbolsLocally(Environment& env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
    static Sets* create();
};

}

#endif
