/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * The SMT-LIB 2 preprocessor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The SMT-LIB 2 preprocessor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the SMT-LIB 2 preprocessor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * General helper functions
 */

#ifndef UTIL
#define UTIL 1

#include "refcounting.H"

#include <vector>
#include <iostream>
#include <functional>
#include <set>

template <class T>
std::vector<T> to_vector(const T& arg0) {
	std::vector<T> res;
	res.push_back(arg0);
	return res;
}

template <class T>
std::vector<T> to_vector(const T& arg0, const T& arg1) {
	std::vector<T> res;
	res.push_back(arg0);
	res.push_back(arg1);
	return res;
}

template <class T>
std::vector<T> to_vector(const T& arg0, const T& arg1, const T& arg2) {
    std::vector<T> res;
    res.push_back(arg0);
    res.push_back(arg1);
    res.push_back(arg2);
    return res;
}

template <class T>
struct print : public std::unary_function<T, void> {
private:
	std::ostream& os;
	const std::string sep;
	bool first;

public:
	print(std::ostream& out, const std::string& sep)
		: os(out), sep(sep), first(true) {}
	void operator() (T x) {
		if (!first)
			os << sep;
		first = false;
		os << x;
	}
};

template <class T>
struct print_deref : public std::unary_function<T, void> {

private:
	std::ostream& os;
	const std::string sep;
	bool first;

public:
	print_deref(std::ostream& out, const std::string& sep)
		: os(out), sep(sep), first(true) {}

	void operator() (T x) {
		if (!first)
			os << sep;
		first = false;
		os << *x;
	}
};

template <class T>
bool disjoint(const std::set<T>& a, const std::set<T>& b) {
    for (typename std::set<T>::iterator it = a.begin();
         it != a.end();
         ++it) {
        if (b.find(*it) != b.end())
            return false;
    }
    return true;
}

template <class T>
void insertAll(std::set<T>& target, const std::set<T>& source) {
    for (typename std::set<T>::const_iterator it = source.begin();
        it != source.end();
        ++it)
        target.insert(*it);
}

/**
 * Simplistic implementation of an immutable singly-linked list (functional
 * datastructures are beautiful)
 */
template <class T>
struct SinglyLinkedList : public RefCounted {
    const T value;
    const CountingPtr<SinglyLinkedList<T> > next;

    SinglyLinkedList(const T& value,
                     const CountingPtr<SinglyLinkedList<T> >& next)
        : value(value), next(next) {}

    CountingPtr<SinglyLinkedList<T> > drop(unsigned int n) {
        SinglyLinkedList<T>* res = this;
        while (n > 0) {
            res = res->next;
            n = n - 1;
        }
        return res;
    }
};

#endif
